@@ -17,11 +17,11 @@ Test::Mock::Redis - use in place of Redis for unit testing
 
 =head1 VERSION
 
-Version 0.14
+Version 0.19
 
 =cut
 
-our $VERSION = '0.14';
+our $VERSION = '0.19';
 
 =head1 SYNOPSIS
 
@@ -55,7 +55,7 @@ returned, with all data preserved.
 
 =head2 new
 
-    Create a new Test::Mock::Redis object. 
+    Create a new Test::Mock::Redis object.
 
     It can be used in place of a Redis object for unit testing.
 
@@ -63,7 +63,7 @@ returned, with all data preserved.
 
 =head2 num_databases
 
-Redis ships with a default of 16 databases, and that's what this module 
+Redis ships with a default of 16 databases, and that's what this module
 handles by default. If you need to change that, do
 
     use Test::Mock::Redis num_databases => 21;
@@ -320,11 +320,14 @@ sub msetnx {
 }
 
 sub del {
-    my ( $self, $key ) = @_;
+    my ( $self, @keys ) = @_;
 
-    my $ret = $self->exists($key);
+    my $ret = 0;
 
-    delete $self->_stash->{$key};
+    for my $key (@keys) {
+        $ret++ if $self->exists($key);
+        delete $self->_stash->{$key};
+    }
 
     return $ret;
 }
@@ -337,15 +340,15 @@ sub type {
 
     my $type = ref $self->_stash->{$key};
 
-    return !$type 
+    return !$type
          ? 'string'
-         : $type eq 'Test::Mock::Redis::Hash' 
+         : $type eq 'Test::Mock::Redis::Hash'
            ? 'hash'
            : $type eq 'Test::Mock::Redis::Set'
              ? 'set'
              : $type eq 'Test::Mock::Redis::ZSet'
                ? 'zset'
-                 : $type eq 'Test::Mock::Redis::List' 
+                 : $type eq 'Test::Mock::Redis::List'
                    ? 'list'
                    : 'unknown'
     ;
@@ -362,7 +365,7 @@ sub keys :method {
 
     return @{[ sort { $a cmp $b }
         grep { exists $self->_stash->{$_} }
-        grep { /$match/ }
+        grep { /^$match/ }
         keys %{ $self->_stash }]};
 }
 
@@ -439,6 +442,16 @@ sub lpushx {
     return scalar @{ $self->_stash->{$key} };
 }
 
+sub rpoplpush {
+    my ( $self, $source_key, $destination_key ) = @_;
+
+    my $popped_element = $self->rpop( $source_key ) or return;
+
+    $self->lpush( $destination_key, $popped_element );
+
+    return $popped_element;
+}
+
 sub llen {
     my ( $self, $key ) = @_;
 
@@ -456,7 +469,7 @@ sub lrange {
 sub ltrim {
     my ( $self, $key, $start, $end ) = @_;
 
-    $self->_stash->{$key} = [ @{ $self->_stash->{$key} }[$start..$end] ]; 
+    $self->_stash->{$key} = [ @{ $self->_stash->{$key} }[$start..$end] ];
     return 'OK';
 }
 
@@ -488,7 +501,7 @@ sub lrem {
             last if $count && ++$removed >= $count;
         }
     }
-    
+
     return $removed;
 }
 
@@ -513,7 +526,7 @@ sub select {
 
     my $max_index = $#{ $self->{_stash} };
     if ($index > $max_index ){
-        die "You called select($index), but max allowed is $max_index unless you configure more databases"; 
+        die "You called select($index), but max allowed is $max_index unless you configure more databases";
     }
 
     $self->{_db_index} = $index;
@@ -550,7 +563,7 @@ sub sismember {
     my ( $self, $key, $value ) = @_;
 
     return exists $self->_stash->{$key}->{$value}
-            ? 1 
+            ? 1
             : 0;
 }
 
@@ -724,7 +737,7 @@ sub hexists {
     confess '[hexists] ERR Operation against a key holding the wrong kind of value'
          if $self->exists($key) and !$self->_is_hash($key);
 
-    return exists $self->_stash->{$key}->{$hkey};
+    return exists $self->_stash->{$key}->{$hkey} ? 1 : 0;
 }
 
 sub hdel {
@@ -826,7 +839,7 @@ sub sort {
 
     my $cmp = do
     { no warnings 'uninitialized';
-      $how =~ /\bALPHA\b/ 
+      $how =~ /\bALPHA\b/
       ? $how =~ /\bDESC\b/
         ? sub { $b cmp $a }
         : sub { $a cmp $b }
@@ -839,18 +852,18 @@ sub sort {
     return sort $cmp @{ $self->_stash->{$key} };
 }
 
-sub save { 
+sub save {
     my $self = shift;
     $self->{_last_save} = time;
     return 'OK';
 }
 
-sub bgsave { 
+sub bgsave {
     my $self = shift;
     return $self->save;
 }
 
-sub lastsave { 
+sub lastsave {
     my $self = shift;
     return $self->{_last_save};
 }
@@ -983,12 +996,12 @@ sub zrange {
     my ( $self, $key, $start, $stop, $withscores ) = @_;
 
     $stop = $self->zcard($key)-1 if $stop >= $self->zcard($key);
-    
-    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ } 
+
+    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
                ( map { $_->[0] }
-                     sort { $a->[1] <=> $b->[1] }
+                     sort { $a->[1] <=> $b->[1] || $a->[0] cmp $b->[0] }
                          map { [ $_, $self->_stash->{$key}->{$_} ] }
-                             keys %{ $self->_stash->{$key} } 
+                             keys %{ $self->_stash->{$key} }
                )[$start..$stop]
     ;
 }
@@ -998,11 +1011,11 @@ sub zrevrange {
 
     $stop = $self->zcard($key)-1 if $stop >= $self->zcard($key);
 
-    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ } 
+    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
                ( map { $_->[0] }
                      sort { $b->[1] <=> $a->[1] }
                          map { [ $_, $self->_stash->{$key}->{$_} ] }
-                             keys %{ $self->_stash->{$key} } 
+                             keys %{ $self->_stash->{$key} }
                )[$start..$stop]
     ;
 }
@@ -1015,16 +1028,25 @@ sub zrangebyscore {
 
     my $cmp = !$min_inc && !$max_inc
             ? sub { $self->zscore($key, $_[0]) > $min && $self->zscore($key, $_[0]) < $max }
-            : !$min_inc 
+            : !$min_inc
               ? sub { $self->zscore($key, $_[0]) > $min && $self->zscore($key, $_[0]) <= $max }
-              : !$max_inc 
+              : !$max_inc
                 ? sub { $self->zscore($key, $_[0]) >= $min && $self->zscore($key, $_[0]) <  $max }
                 : sub { $self->zscore($key, $_[0]) >= $min && $self->zscore($key, $_[0]) <= $max }
     ;
-            
-    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ } 
+
+    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
                grep { $cmp->($_) } $self->zrange($key, 0, $self->zcard($key)-1);
-                   $self->zrange($key, 0, $self->zcard($key)-1);
+}
+
+# note max and min are reversed from zrangebyscore
+sub zrevrangebyscore {
+    my ( $self, $key, $max, $min, $withscores ) = @_;
+
+    my $not_with_scores = 0;
+
+    return map { $withscores ? ( $_, $self->zscore($key, $_) ) : $_ }
+               reverse $self->zrangebyscore($key, $min, $max, $not_with_scores);
 }
 
 sub zcount {
@@ -1142,15 +1164,25 @@ The following people have contributed to I<Test::Mock::Redis>:
 
 =over
 
+=item * Chris Reinhardt
+
+=item * Ian Burrell
+
 =item * Karen Etheridge
 
+=item * Keith Broughton
+
 =item * Kevin Goess
 
+=item * Neil Bowers
+
+=item * Yaakov Shaul
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011, 2012, 2013 Jeff Lavallee.
+Copyright 2015 Jeff Lavallee.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
@@ -1280,6 +1312,22 @@ sub discard {
     return 'OK';
 }
 
+sub watch {
+    my ($self) = shift;
+
+    confess '[watch] ERR wrong number of arguments for \'watch\' command' unless @_;
+
+    return 'OK';
+}
+
+sub unwatch {
+    my ($self) = shift;
+
+    confess '[error] ERR wrong number of arguments for \'unwatch\' command' if @_;
+
+    return 'OK';
+}
+
 # now that we've defined all our subs, we need to wrap them all in logic that
 # can check if we are in the middle of a MULTI, and if so, queue up the
 # commands for later replaying.
@@ -1414,7 +1462,7 @@ use strict; use warnings;
 use Tie::Hash;
 use base qw/Tie::StdHash/;
 
-sub DELETE { 
+sub DELETE {
     my ( $self, $key ) = @_;
 
     delete $self->{$key};
@@ -1475,4 +1523,3 @@ sub ttl {
 
 
 1;
-
